#include "mas35x9_usb.h"

#include "misc.h"

#define IF_DO_NOT_EQUAL_RETRUN_NULL( a, b) if((a) != (b)) return NULL;



void* mas35x9_probe(
			struct usb_device *udev,
			unsigned int ifnum,
			const struct usb_device_id *id )
{
	printk("call mas35x9_probe\n");

	printk("======================================\n");
	printk("[ check usb_device_id ]\n");
	printk("usb_device_id->idVendor : %x\n", id->idVendor);
	printk("usb_device_id->idProduct : %x\n", id->idProduct);
	printk("usb_device_id->bDeviceClass : %x\n", id->bDeviceClass);
	printk("usb_device_id->bDeviceSubClass : %x\n", id->bDeviceSubClass);
	printk("usb_device_id->bDeviceProtocol : %x\n", id->bDeviceProtocol);
	printk("usb_device_id->bInterfaceClass : %x\n", id->bInterfaceClass);
	printk("usb_device_id->bInterfaceProtocol : %x\n", id->bInterfaceProtocol);
	printk("======================================\n");

	printk("[ start check ]\n");
	IF_DO_NOT_EQUAL_RETRUN_NULL(udev->descriptor.idVendor, id->idVendor);
	printk("check ok : idVendor\n");
	IF_DO_NOT_EQUAL_RETRUN_NULL(udev->descriptor.idProduct, id->idProduct);
	printk("check ok : idProduct\n");
	IF_DO_NOT_EQUAL_RETRUN_NULL(udev->descriptor.bDeviceClass, id->bDeviceClass);
	printk("check ok : bDeviceClass\n");
	IF_DO_NOT_EQUAL_RETRUN_NULL(udev->descriptor.bDeviceSubClass, id->bDeviceSubClass);
	printk("check ok : bDeviceSubClass\n");
	IF_DO_NOT_EQUAL_RETRUN_NULL(udev->descriptor.bDeviceProtocol, id->bDeviceProtocol);
	printk("check ok : bDeviceProtocol\n");
	printk("======================================\n");
	printk("usb_device_id fields check ALL OK \n");
	printk("======================================\n");

	if(!proc_init()){
		return NULL;
	}


//	gMAS35x9 = (struct MAS35x9 *)kmalloc(sizeof(struct MAS35x9), GFP_KERNEL);

	
	gMAS35x9.udev		= udev;
	gMAS35x9.interface = &(udev->actconfig->interface[ifnum]);
	gMAS35x9.iface_desc= &gMAS35x9.interface->altsetting[0];

	printk("--------------------------------------\n");
	printk("Number of Endpoint : %d\n", gMAS35x9.iface_desc->bNumEndpoints);
	printk("--------------------------------------\n");

	gMAS35x9.ep0_desc	= &(gMAS35x9.iface_desc->endpoint[0]);
	gMAS35x9.ep1_desc	= &(gMAS35x9.iface_desc->endpoint[1]);
	gMAS35x9.ep2_desc	= &(gMAS35x9.iface_desc->endpoint[2]);
	gMAS35x9.ep3_desc	= &(gMAS35x9.iface_desc->endpoint[3]);

	print_endpoint_status(gMAS35x9.ep0_desc, (unsigned int)0);
	print_endpoint_status(gMAS35x9.ep1_desc, (unsigned int)1);
	print_endpoint_status(gMAS35x9.ep2_desc, (unsigned int)2);
	print_endpoint_status(gMAS35x9.ep3_desc, (unsigned int)3);

	return &gMAS35x9;
}


void mas35x9_disconnect(struct usb_device *udev, void *v)
{
	printk("call mas35x9_disconnect. \n");

//	MOD_DEC_USE_COUNT;


	proc_exit();
}


int __init module_mas35x9_init()
{
	printk("call usb_register :)\n");
	usb_register(&mas35x9_usb_driver);


	return 0;
}


void __exit module_mas35x9_exit()
{
	printk("call usb_deregister :)\n");
	usb_deregister(&mas35x9_usb_driver);
}

int mas35x9_proc_info_init()
{
	s8		MAS35x9_firmware_version = '\0';
	s32		MAS35x9_buffersize = 0;

	SMC_INFO	SMC_info = {0, 0, 0, 0};
	s16			SMC_root_block = 0;

	int pos = 0;

	/*	VENDOR_CMD_GET_FIRMWARE_VERSION,1,
        VENDOR_CMD_GET_BUFFER_SIZE,     4,
        VENDOR_CMD_GET_SMC_INFO,	    5,
        VENDOR_CMD_GET_ROOT_BLOCK,		2,	*/

	info_crnt_pos = 0;
	memset(info_str, '\0', INFO_STR_SIZE);

	//   mas35x9_vendor_cmd_read()  ° ڷ Ѱָ
	// 尡 , ׷,   ϰ,  о  ,  
	mas35x9_vendor_cmd_read(VENDOR_CMD_GET_FIRMWARE_VERSION,	1,	&MAS35x9_firmware_version);
	mas35x9_vendor_cmd_read(VENDOR_CMD_GET_BUFFER_SIZE,	4,	&MAS35x9_buffersize);
	mas35x9_vendor_cmd_read(VENDOR_CMD_GET_SMC_INFO,	5,	&SMC_info);
	mas35x9_vendor_cmd_read(VENDOR_CMD_GET_ROOT_BLOCK,	2,	&SMC_root_block);

	pos  = sprintf(info_str, "MAS35x9 Firmware Version : %d\n", MAS35x9_firmware_version);
	pos += sprintf(info_str +pos, "MAS35x9 buffer size : %d\n", MAS35x9_buffersize);

	pos += sprintf(info_str +pos, "SMC Manufacturer : 0x%X\n", SMC_info.manufacturer);
	pos += sprintf(info_str +pos, "SMC Size : 0x%X\n", SMC_info.size);
	pos += sprintf(info_str +pos, "SMC Block Count : %d\n", SMC_info.block_cnt);
	pos += sprintf(info_str +pos, "SMC Block/Page : %d\n", SMC_info.page_per_block);
	pos += sprintf(info_str +pos, "SMC Root Block Number : %d\n", SMC_root_block);

	memcpy(&gSMC_info,		 &SMC_info,			sizeof(SMC_info));
	memcpy(&gSMC_root_block, &SMC_root_block,	sizeof(SMC_root_block));

	return pos;
}


int mas35x9_vendor_cmd_read(u16 index,	u16 size,	void* data)
{
	int urb_error = 0;

	urb_error = usb_control_msg(
					gMAS35x9.udev,
					usb_rcvctrlpipe(gMAS35x9.udev, 0) ,
					REQUEST,
					(USB_DIR_IN | USB_TYPE_VENDOR),
					VALUE, // wValue
					index, //wIndex  
					data,	// snd pipe  쿡  
							// rcv pipe  쿡  
					size,  // wLength
							//   data 
					TIMEOUT);

	return urb_error;
}



int process_urb_error(int error)
{
	if(error < 0)
	{
		switch(error)
		{
			case -ENOMEM:
				printk("-ENOMEM\n");
				break;

			default:
				printk("other error\n");
				break;
		}
	}

	return error;
}



/*
int proc_tmp_write(struct file *file, const char *buf, unsigned long count, void *data)
{
	int readed_length = 0;
	u8	ep2_rcv_data[512+16] = {'x'};

	if(count !=4){
		printk("tmp count is not 4\n");
		return -1;
	}

	readed_length = smc_read_page_A_area(64, ep2_rcv_data);
	dump_8(ep2_rcv_data, 256);

	readed_length = smc_read_page_B_area(64, ep2_rcv_data+256);
	dump_8(ep2_rcv_data+256, 256);

	readed_length = smc_read_page_C_area(64, ep2_rcv_data+512);
	dump_8(ep2_rcv_data+512, 16);

	return 4;
}
*/

int mas35x9_vendor_cmd_write(u16 index,	u16 size,	void* data)
{
	int urb_error = 0;

	urb_error = usb_control_msg(
					gMAS35x9.udev,
					usb_sndctrlpipe(gMAS35x9.udev, 0) ,
					REQUEST,
					(USB_DIR_OUT | USB_TYPE_VENDOR),
					VALUE, // wValue
					index, //wIndex  
					data,
					size,  // wLength
							//   data 
					TIMEOUT);

	return urb_error;
}



/*

offset : 0, 255, 512    ̾ 
		 ٸ   尡 ٿ Ǵ° 
*/
int smc_read_page(u32 page, u16 offset, u16 len, void* data)
{
	int urb_error = 0;
	int actual_length = 0;

	int i=0;
	int length_to_read = 0;
	int rtn_readed_length = 0;

	SMC_REQUEST	smc_request;
	IO_REQUEST io_request;

	printk("before READ_SMC_PAGE\n");

	smc_request.dwPage  = page;
	smc_request.wOffset = offset;
	smc_request.wLen    = len;
	urb_error = mas35x9_vendor_cmd_write(VENDOR_CMD_READ_SMC_PAGE, 8, &smc_request);
	if( ERROR(process_urb_error(urb_error)) ){
		return -1;
	}

	printk("after READ_SMC_PAGE\n");

	io_request.wAddressL = 0;
	io_request.cAddressH = 0;
	io_request.wSize     = smc_request.wLen;
	io_request.cCommand  = 0x01;

	printk("before SETUP_DMA\n");

	urb_error = mas35x9_vendor_cmd_write(VENDOR_CMD_SETUP_DMA_REQUEST, 6, &io_request);
	if( ERROR(process_urb_error(urb_error)) ){
		return -1;
	}

	printk("after SETUP_DMA\n");

	length_to_read = smc_request.wLen;

	for(i=0; length_to_read>0 ; ++i)
	{
		printk("before rcv [%d]\n", i);

		urb_error = usb_bulk_msg(
						gMAS35x9.udev,
						usb_rcvbulkpipe(gMAS35x9.udev, 2),
						(data+ i*EP2_PACKET_LENGTH),
						EP2_PACKET_LENGTH,
						&actual_length,
						TIMEOUT);

		printk("after rcv [%d]\n", i);

		if( ERROR(process_urb_error(urb_error)) ){
			return -1;
		}

		length_to_read -= EP2_PACKET_LENGTH;
		rtn_readed_length += actual_length;
	}

	return rtn_readed_length;
}




