#include <linux/module.h>
#include <linux/init.h>
#include <linux/slab.h>		// kmalloc, kfree
#include <asm/uaccess.h>	// copy_from_user

#include <linux/usb.h>
#include <linux/proc_fs.h>


#include "mas35x9_cmd.h"

#define	FALSE	0
#define	TRUE	1


EXPORT_NO_SYMBOLS;

MODULE_AUTHOR("Kim ByungKun (skysign@dreamwiz.com)");
MODULE_DESCRIPTION("MAS35x9 MP3 Player USB Driver for Linux [http://www.wavysound.com]");
MODULE_LICENSE("GPL");


char mas35x9_name[] = "[MAS35x9 USB Driver for Linux]";

/* Get a minor range for your devices from the usb maintainer */
#define USB_MAS35x9_MINOR_BASE	1
/* we can have up to this number of device plugged in at once */
#define MAX_DEVICES		1

#define	NOERROR(x) (x)>=0
#define	ERROR(x) (x)<0

#define MAS35x9_USB_VENDOR_ID	0x0471
#define MAS35x9_USB_PRODUCT_ID	0x0888








struct MAS35x9
{
	struct usb_device *					udev;
	struct usb_interface *				interface;
	struct usb_interface_descriptor *	iface_desc;

	struct usb_endpoint_descriptor *	ep0_desc;
	struct usb_endpoint_descriptor *	ep1_desc;
	struct usb_endpoint_descriptor *	ep2_desc;
	struct usb_endpoint_descriptor *	ep3_desc;

	int	bConnected;
};

struct MAS35x9	gMAS35x9;





int  __init module_mas35x9_init();
void __exit module_mas35x9_exit();
void* mas35x9_probe(struct usb_device *udev,  unsigned intf,  const struct usb_device_id *id );
void  mas35x9_disconnect(struct usb_device *udev, void *v);



static struct usb_device_id mas35x9_usb_device_id [] = {
	{ USB_DEVICE(MAS35x9_USB_VENDOR_ID, MAS35x9_USB_PRODUCT_ID),
	  bDeviceClass    :		0x08,
	  bDeviceSubClass :		0x00,
	  bDeviceProtocol :		0x00,
	  bInterfaceClass :		0x08,
	  bInterfaceSubClass :	0x01,
	  bInterfaceProtocol :	0x00
	},
	{ }					/* Terminating entry */
};

MODULE_DEVICE_TABLE (usb, mas35x9_usb_device_id);



static struct file_operations mas35x9_fops = {
	owner:		THIS_MODULE
};

static struct usb_driver mas35x9_usb_driver = {
	name:		mas35x9_name,
	probe:		mas35x9_probe,
	disconnect:	mas35x9_disconnect,
	fops:		&mas35x9_fops,
	minor:		USB_MAS35x9_MINOR_BASE,
	id_table:	mas35x9_usb_device_id,
};



// These Two function need to include "linux/init.h"
module_init( module_mas35x9_init );
module_exit( module_mas35x9_exit );



int process_urb_error(int error);


int mas35x9_vendor_cmd_read(u16 index,	u16 size,	void* data);
int mas35x9_vendor_cmd_write(u16 index,	u16 size,	void* data);


#define	INFO_STR_SIZE	1024
char info_str[INFO_STR_SIZE];
int	 info_crnt_pos;

int mas35x9_proc_info_init();

SMC_INFO	gSMC_info = {0, 0, 0, 0};
s16			gSMC_root_block = 0;

#define	smc_read_page_A_area(page, pdata) smc_read_page(page, 0, 256, pdata);
#define	smc_read_page_B_area(page, pdata) smc_read_page(page, 255, 256, pdata);
#define	smc_read_page_C_area(page, pdata) smc_read_page(page, 511, 16, pdata);

int smc_read_page(u32 page, u16 offset, u16 len, void* data);


extern int	proc_init();
extern void	proc_exit();

int proc_tmp_write(struct file *file, const char *buf, unsigned long count, void *data);

































void print_endpoint_status(struct usb_endpoint_descriptor* ep, unsigned int endpoint_number)
{
	printk("info of End Point : %d\n", endpoint_number);
	printk("-----------------------------------------\n");
	printk("bLength			: %d\n", ep->bLength);
	printk("bDescriptorType : %d\n", ep->bDescriptorType);

	printk("bEndpointAddress: %d\n", ep->bEndpointAddress);
	switch(ep->bEndpointAddress & 0x80){	// direction
		case 0x00:
			printk("   RCV Device->PC");
			break;

		case 0x80:
			printk("   SND PC->Device");
			break;

		default:
			printk("    bEndpointAddress error \n");
			break;
	}
	printk(" EP Num %d\n", ep->bEndpointAddress & 0x07);

	printk("bmAttributes	: %d\n", ep->bmAttributes);
	switch(ep->bmAttributes & 0x03){
		case 0x00:
			printk("    Control\n");
			break;

		case 0x01:
			printk("    Async\n");
			break;

		case 0x02:
			printk("    Bulk\n");
			break;

		case 0x03:
			printk("    Interrupt\n");
			break;

		default:
			printk("    bmAttributes error \n");
			break;
	}

	printk("wMaxPacketSize	: %d\n", ep->wMaxPacketSize);
	printk("bInterval		: %d\n", ep->bInterval);
	printk("bRefresh		: %d\n", ep->bRefresh);
	printk("bSynchAddress	: %d\n", ep->bSynchAddress);
	printk("=========================================\n");
}



void dump_8(void* data, int len)
{
	int i = 0;
	int j = 0;
	int limit = 0;

	char*	p = (char*)data;

	
	for(j=0; j<(len/8); ++j)
	{
		limit = (j+1)*8 < len? (j+1)*8 : len;

		for(i= j*8; i<limit ; ++i)
		{
			printk("%4d ", p[i]);
		}

		printk("     ");

		printk("[");
		for(i= j*8; i<limit; ++i)
		{
			switch(p[i])
			{
				case '\0':
					printk(" ");
				break;

				default:
					printk("%c", p[i]);
				break;
			}

		}
		printk("]");

		printk("\n");
	}
}




